package org.jutils.corba;
/*------------------------------------------------------------------------------
     Name: CorbaHelper.java
  Project: jutils
  Version: $Revision: 1.1 $
    Datum: $Date: 2001/05/23 07:51:27 $
   Author: oliver.belikan@doubleslash.de
  Comment: Corba helper class
--------------------------------------------------------------------------------*/

import java.io.*;
/**
 * Delivering the IOR over HTTP.
 * <p />
 * This little HTTP server is used to get the iorString on every
 * userrequest. As an example. http://localhost:port/AuthenticationServer.ior<br />
 * Clients may access through this port the IOR if they
 * don't want to use a naming service. The default port ist 7109.
 */
public class CorbaHelper {
   /************************************************************
  Converts the CORBA <code>any</code> type to the JAVA <code>byte[]</code> type.
  <br><b>Beispiel:</b><br>
    <code>Utils.any2byte(someAny);</code><br>
    -> returns a byte array containg the (binary) data
  @param <code>any</code> from org.omg.CORBA.ORB.init().create_any()
  @return <code>byte[]</code>
  @see #byte2any(byte[])
  ************************************************************/
  public final static byte[] any2byte(org.omg.CORBA.Any value)
  {
    try
    {
      org.omg.CORBA.portable.OutputStream ostream;
      ostream = org.omg.CORBA.ORB.init().create_output_stream();
      ostream.write_any(value);

      org.omg.CORBA.portable.InputStream istream;
      istream = ostream.create_input_stream();

      int len = istream.available();
      byte b[] = new byte[len];
      istream.read(b);

      return b;
    }
    catch(IOException e)
    {
      //Log.error(" Can't convert CORBA-any to byte[]: " + e.toString());
      return new byte[0];
    }
  }

  /************************************************************
  This static method converts the JAVA <code>byte[]</code> type to the
  CORBA <code>any</code> type
  <br><b>Beispiel:</b><br>
    <code>Utils.byte2any(myBytes)</code>
    -> returns a CORBA <code>any</code>
  @param b         some (binary) data
  @return
         any       from org.omg.CORBA.ORB.init().create_any()
  ************************************************************/
  public final static org.omg.CORBA.Any byte2any(byte[] b)
  {
    try
    {
      org.omg.CORBA.portable.OutputStream ostream;
      ostream = org.omg.CORBA.ORB.init().create_output_stream();
      ostream.write(b);
      // if(DEBUG) Log.trace("  byte2any(): ostream filled by byte[]");

      org.omg.CORBA.portable.InputStream istream;
      istream = ostream.create_input_stream();
      // if(DEBUG) Log.trace("  byte2any(): istream created");

      return istream.read_any();
    }
    catch(IOException e)
    {
      //Log.error(" Can't convert byte[] to CORBA-any: " + e.toString());
      return org.omg.CORBA.ORB.init().create_any();
    }
  }
}
