/*------------------------------------------------------------------------------
     Name: HttpIORServer.java
  Project: jutils
  Version: $Revision: 1.7 $
    Datum: $Date: 2001/08/06 08:34:35 $
   Author: oliver.belikan@doubleslash.de
  Comment: Corba helper class
--------------------------------------------------------------------------------*/
package org.jutils.corba;

import java.util.*;
import java.net.*;
import java.io.*;

import org.jutils.log.LogManager;
import org.jutils.log.*;
import org.jutils.init.Property;
import java.util.StringTokenizer;

/**
 * Delivering the IOR over HTTP.
 * <p />
 * This little HTTP server is used to get the iorString on every
 * userrequest. As an example. http://localhost:port/AuthenticationServer.ior<br />
 * Clients may access through this port the IOR if they
 * don't want to use a naming service. The default port ist 7109.
 */
public class HttpIORServer
   extends Thread
{
   private String ME                      = "HttpIORServer";   // this is the name of me
   private int HTTP_PORT                  = 7109;            // default port
   private String ior                     = null;             // the internal generated string given at startup
   private ServerSocket listen            = null;
   private boolean running                = true;

   protected static String LCN            = null;    // Name of the logchannel
   private Property prop                  = null;          // Property read from jutils
   public String pathToIORFile            = null;
   public String IORservicename           = "";


   /**
    * Create an new instance on default port 7109.
    *
    * @param      ior
    *             The identifying object id.
    */
   public HttpIORServer (String ior) {
      this.ior = ior;
      // switch on logging to console
      LogChannel lc = new LogChannel(null);
      LogDeviceConsole ldc = new LogDeviceConsole(lc);
      lc.setLogLevel(LogChannel.LOG_ALL);
      lc.addLogDevice(ldc);
      LogManager.addLogChannel(ME, lc);
      this.LCN = ME;
      LogManager.call(LCN, ME, "Creating new HttpIORServer with ior = " + ior + " on port " + HTTP_PORT);
      start();
   }

   /**
    * Create an new instance.
    *
    * @param      port
    *             The port which this server will run.
    * @param      ior
    *             The identifying object id.
    */
   public HttpIORServer (int port, String ior, String servicename)
   {
      this.ior = ior;
      this.HTTP_PORT = port;
      this.IORservicename = servicename;
      // switch on logging to console
      LogChannel lc = new LogChannel(null);
      LogDeviceConsole ldc = new LogDeviceConsole(lc);
      lc.setLogLevel(LogChannel.LOG_ALL);
      lc.addLogDevice(ldc);
      LogManager.addLogChannel(ME, lc);
      this.LCN = ME;
      LogManager.call(LCN, ME, "Creating new HttpIORServer with ior = " + ior + " on port " + port);
      start();
   }
   /**
    * Create an new instance.
    *
    * @param      port
    *             The port which this server will run.
    * @param      ior
    *             The identifieing object id.
    * @param      LCN
    *             Name of the channel where to log to (initialized witd LogChannel).
    * @param      servicename
    *             Name of the service wich is to be serverd ex. authserver.ior
    */
   public HttpIORServer (int port, String ior, String LCN, String servicename)
   {
      this.ior = ior;
      this.HTTP_PORT = port;
      this.LCN = LCN;
      this.IORservicename = servicename;
      LogManager.call(LCN, ME, "Creating new HttpIORServer with ior = " + ior + " on port " + port);
      start();
   }

   /**
    * sets the path to the directory from
    * where the ior files will be served.
    * the files ending is .ior
    */
   public void setPath (String newPath) {
      this.pathToIORFile = newPath;
   }
   /**
    * The thread starts here and handles every request with
    * an thread over HandleRequest.
    */
   public void run()
   {
     int backlog = 50; // queue for max 50 incoming connection request
      try {
         listen = new ServerSocket(HTTP_PORT, backlog);
         while (running) {
            Socket accept = listen.accept();
            if (!running) {
               LogManager.info(LCN, ME, "Closing http server port=" + HTTP_PORT + ".");
               break;
            }
            HandleRequest hh = new HandleRequest (accept, ior, pathToIORFile, IORservicename);
         }
      }
      catch (java.net.BindException e) {
         LogManager.error(LCN, ME, "HTTP server problem: " + e.toString());
      }
      catch (java.net.SocketException e) {
         LogManager.info(LCN, ME, "Socket closed successfully: " + e.toString());
      }
      catch (IOException e) {
         LogManager.error(LCN, ME, "HTTP server problem: " + e.toString());
      }

      if (listen != null) {
         try { listen.close(); } catch (java.io.IOException e) { LogManager.warn(LCN, ME, "listen.close()" + e.toString()); }
         listen = null;
      }
   }


   /**
    * Close the listener port and stops this Httpserver.
    */
   public void shutdown()// throws IOException
   {
      LogManager.call(LCN, ME, "Entering shutdown");
      running = false;

      boolean closeHack = true;

      if (listen != null && closeHack) {

         try {
            java.net.Socket socket = new Socket(listen.getInetAddress(), HTTP_PORT);
            socket.close();
         } catch (java.io.IOException e) {
            LogManager.warn(LCN, ME, "shutdown problem: " + e.toString());
         }
      }
      try {
         if (listen != null) {
            listen.close();
            listen = null;
         }
      } catch (Exception e) {
         LogManager.warn(LCN, ME, "die zweite problem: " + e.toString());
      }
   }


   /**
    * Just for testing
    */
   public static void main( String[] args )
   {
      HttpIORServer myServer = new HttpIORServer(6001, "IOR:098080809809808", "authserver.ior");
      HttpIORClient myClient = new HttpIORClient ();
      String ior = myClient.getIORFromUrl ("http://localhost:6001/authserver.ior");
      System.out.println("received ior = " + ior);
      ior = myClient.getIORFromUrl ("http://localhost:6001/test.ior");
      System.out.println("received ior = " + ior);
      try{
         Thread.sleep(20000);
      }
      catch(InterruptedException ie) {}
         myServer.shutdown();
   }
}

/**
 * Handles a request from a client, delivering the AuthServer IOR
 */
class HandleRequest
   extends Thread
{
   private String ME = "HandleRequest";
   private final Socket sock;
   private final String ior;
   private final String CRLF = "\r\n";
   private String pathToIORFile = "";
   private String IORservicename = "";


   /**
    * HandleRequest
    */
   public HandleRequest(Socket sock, String iorStr, String pathToIORFile, String servicename)
   {
      this.sock = sock;
      this.ior = iorStr;
      this.IORservicename = servicename;
      this.pathToIORFile = pathToIORFile;
      start();
   }


   /**
    */
   public void run()
   {
      LogManager.call(HttpIORServer.LCN, ME, "Handling client request, accessing server IOR ...");
      BufferedReader iStream = null;
      DataOutputStream oStream = null;
      try {
         iStream = new BufferedReader(new InputStreamReader(sock.getInputStream()));
         oStream = new DataOutputStream(sock.getOutputStream());

         String clientRequest = iStream.readLine(); // "GET /name.ior HTTP/1.0"
         iStream.readLine(); // "\r\n"
         LogManager.trace(HttpIORServer.LCN, ME, "clientrequest = '" + clientRequest +"'");

         if (clientRequest == null) {
            String code = "HTTP/1.1 400 Bad Request";
            oStream.write((code+CRLF).getBytes());
            oStream.write(("Server: Doubleslash-WebServer/1.0 name.ior/1.0"+CRLF).getBytes());
            oStream.write(("Connection: close"+CRLF).getBytes());
            oStream.write((CRLF+"<html><head><title>"+code+"</title></head><body>"+code+"</body></html>").getBytes());
            LogManager.warn (HttpIORServer.LCN, ME, "Empty client request");
            return;
         }

         StringTokenizer toks = new StringTokenizer(clientRequest);
         if (toks.countTokens() != 3) {
            String code = "HTTP/1.1 400 Bad Request";
            oStream.write((code+CRLF).getBytes());
            oStream.write(("Server: Doubleslash-WebServer/1.0 name.ior/1.0"+CRLF).getBytes());
            oStream.write(("Connection: close"+CRLF).getBytes());
            oStream.write((CRLF+"<html><head><title>"+code+"</title></head><body>"+code+"</body></html>").getBytes());
            LogManager.warn (HttpIORServer.LCN, ME, "Wrong syntax in client request: '" + clientRequest + "'");
            return;
         }

         String method = toks.nextToken();   // "GET"
         String resource = toks.nextToken(); // "/name.ior"
         String version = toks.nextToken();  // "HTTP/1.0"

         // RFC 2068 enforces minimum implementation GET and HEAD
         if (!method.equalsIgnoreCase("GET") && !method.equalsIgnoreCase("HEAD")) {
            String code = "HTTP/1.1 501 Method Not Implemented";
            oStream.write((code+CRLF).getBytes());
            oStream.write(("Server: Doubleslash-WebServer/1.0 name.ior/1.0"+CRLF).getBytes());
            oStream.write(("Allow: GET"+CRLF).getBytes());
            oStream.write(("Connection: close"+CRLF).getBytes());
            oStream.write((CRLF+"<html><head><title>"+code+"</title></head><body>"+code+"</body></html>").getBytes());
            LogManager.warn (HttpIORServer.LCN, ME, "Invalid method in client request: '" + clientRequest + "'");
            return;
         }

         if (resource.equalsIgnoreCase("/" + IORservicename)) {
            oStream.write(("HTTP/1.1 200 OK"+CRLF).getBytes());
            oStream.write(("Server: Doubleslash-WebServer/1.0 "+ IORservicename +"/1.0"+CRLF).getBytes());
            oStream.write(("Connection: close"+CRLF).getBytes());
            String length = "Content-Length: " + ior.length();
            oStream.write((length+CRLF).getBytes());
            oStream.write(("Transfer-Encoding: chunked"+CRLF).getBytes());
            oStream.write(("Content-Type: text/plain; charset=iso-8859-1"+CRLF).getBytes());
            if (!method.equalsIgnoreCase("HEAD")) {
               oStream.write(CRLF.getBytes());
               oStream.write(ior.getBytes());
            }
            return;
         }




         /*
         String filename = pathToIORFile + resource + ".ior";
         File myFile = new File(filename);
         if ( myFile.isFile() ) {
            String fileior = "";
            try {
               fileior = org.jutils.io.FileUtil.readAsciiFile (filename);
            }
            catch (Exception e) {
               LogManager.error (HttpIORServer.LCN , ME, e.toString());
            }
            oStream.write(("HTTP/1.1 200 OK"+CRLF).getBytes());
            oStream.write(("Server: Doubleslash-WebServer/1.0 name.ior/1.0"+CRLF).getBytes());
            oStream.write(("Connection: close"+CRLF).getBytes());
            String length = "Content-Length: " + fileior.length();
            oStream.write((length+CRLF).getBytes());
            oStream.write(("Transfer-Encoding: chunked"+CRLF).getBytes());
            oStream.write(("Content-Type: text/plain; charset=iso-8859-1"+CRLF).getBytes());
            if (!method.equalsIgnoreCase("HEAD")) {
               oStream.write(CRLF.getBytes());
               oStream.write(fileior.getBytes());
            }
         }
         else {
            */
            String code = "HTTP/1.1 404 Not Found";
            oStream.write((code+CRLF).getBytes());
            oStream.write(("Server: Doubleslash-WebServer/1.0 name.ior/1.0"+CRLF).getBytes());
            oStream.write(("Connection: close"+CRLF).getBytes());
            oStream.write((CRLF+"<html><head><title>"+code+"</title></head><body>"+code+"</body></html>").getBytes());
            LogManager.warn (HttpIORServer.LCN, ME, "Ignoring unknown data from client request: '" + clientRequest + "'");
         /*
         }
         return;
         */


      }
      catch (IOException e) {
         LogManager.error (HttpIORServer.LCN, ME, "Problems with sending IOR to client: " + e.toString());
      }
      finally {
         try { if (iStream != null) iStream.close(); } catch (IOException e) { }
         try { if (oStream != null) oStream.close(); } catch (IOException e) { }
         try { sock.close();  } catch (IOException e) { }
      }

   }
}
