package org.jutils.media;
/*------------------------------------------------------------------------------
Name:      MediaHelper.java
Project:   jutils.org
Copyright: jutils.org, see jutils-LICENSE file
Comment:   
------------------------------------------------------------------------------*/

import java.util.Enumeration;
import java.util.Vector;

import org.jutils.JUtilsException;


/******************************************************************************
@author  Juergen Freidling
******************************************************************************/
public class MediaHelper
{
   public final static String MIME_UNKNOWN = "application/octet-stream";
   public final static String EXTENSION_UNKNOWN = "unknown";
   protected final static String[] EXTENSIONS_UNKNOWN = {EXTENSION_UNKNOWN};


   private static String[][][] dataTable = 
      {
         {  {"application/activemessage"},     {""}}, 
         {  {"application/andrew-inset"},      {"ez"}}, 
         {  {"application/applefile"},         {""}}, 
         {  {"application/atomicmail"},        {""}}, 
         {  {"application/cu-seeme"},          {"csm"}}, 
         {  {"application/cu-seeme"},          {"cu"}}, 
         {  {"application/dca-rft"},           {""}}, 
         {  {"application/dec-dx"},            {""}}, 
         {  {"application/excel"},             {"xls"}}, 
         {  {"application/ghostview"},         {""}}, 
         {  {"application/mac-binhex40"},      {"hqx"}}, 
         {  {"application/mac-compactpro"},    {"cpt"}}, 
         {  {"application/macwriteii"},        {""}}, 
         {  {"application/msword"},            {"doc", "dot", "wrd"}}, 
         {  {"application/news-message-id"},   {""}}, 
         {  {"application/news-transmission"}, {""}}, 
         {  {"application/octet-stream"},      {"bin", "dms", "lha", "lzh", "exe", "class"}}, 
         {  {"application/oda"},               {"oda"}}, 
         {  {"application/pdf"},               {"pdf"}}, 
         {  {"application/pgp"},               {"pgp"}}, 
         {  {"application/pgp-signature"},     {"pgp"}}, 
         {  {"application/postscript"},        {"ps", "eps", "ai"}}, 
         {  {"application/powerpoint"},        {"ppt"}}, 
         {  {"application/remote-printing"},   {""}}, 
         {  {"application/rtf"},               {"rtf"}}, 
         {  {"application/slate"},             {""}}, 
         {  {"application/wita"},              {""}}, 
         {  {"application/wordperfect5.1"},    {"wp5"}}, 
         {  {"application/x-123"},             {"wk"}}, 
         {  {"application/x-Wingz"},           {"wz"}}, 
         {  {"application/x-bcpio"},           {"bcpio"}}, 
         {  {"application/x-cdlink"},          {"vcd"}}, 
         {  {"application/x-chess-pgn"},       {"pgn"}}, 
         {  {"application/x-compress"},        {"z", "Z"}}, 
         {  {"application/x-cpio"},            {"cpio"}}, 
         {  {"application/x-csh"},             {"csh"}}, 
         {  {"application/x-debian-package"},  {"deb"}}, 
         {  {"application/x-director"},        {"dcr", "dir", "dxr"}}, 
         {  {"application/x-dvi"},             {"dvi"}}, 
         {  {"application/x-gtar"},            {"gtar", "tgz"}}, 
         {  {"application/x-gunzip"},          {"gz"}}, 
         {  {"application/x-gzip"},            {"gz"}}, 
         {  {"application/x-hdf"},             {"hdf"}}, 
         {  {"application/x-httpd-php"},       {"phtml", "pht", "php"}}, 
         {  {"application/x-javascript"},      {"js"}}, 
         {  {"application/x-koan"},            {"skp", "skd", "skt", "skm"}}, 
         {  {"application/x-latex"},           {"latex"}}, 
         {  {"application/x-maker"},           {"frm", "maker", "frame", "fm", "fb", "book", "fbdoc"}}, 
         {  {"application/x-mif"},             {"mif"}}, 
         {  {"application/x-msdos-program"},   {"com", "exe", "bat"}}, 
         {  {"application/x-netcdf"},          {"nc", "cdf"}}, 
         {  {"application/x-perl"},            {"pl", "pm"}}, 
         {  {"application/x-sh"},              {"sh"}}, 
         {  {"application/x-shar"},            {"shar"}}, 
         {  {"application/x-stuffit"},         {"sit"}}, 
         {  {"application/x-sv4cpio"},         {"sv4cpio"}}, 
         {  {"application/x-sv4crc"},          {"sv4crc"}}, 
         {  {"application/x-tar"},             {"tar"}}, 
         {  {"application/x-tcl"},             {"tcl"}}, 
         {  {"application/x-tex"},             {"tex"}}, 
         {  {"application/x-texinfo"},         {"texinfo", "texi"}}, 
         {  {"application/x-troff"},           {"t", "tr", "roff"}}, 
         {  {"application/x-troff-man"},       {"man"}}, 
         {  {"application/x-troff-me"},        {"me"}}, 
         {  {"application/x-troff-ms"},        {"ms"}}, 
         {  {"application/x-ustar"},           {"ustar"}}, 
         {  {"application/x-wais-source"},     {"src"}}, 
         {  {"application/zip"},               {"zip"}}, 
         {  {"audio/basic"},                   {"au", "snd"}}, 
         {  {"audio/midi"},                    {"mid", "midi", "kar"}}, 
         {  {"audio/mpeg"},                    {"mp3", "mpga", "mp2"}}, 
         {  {"audio/x-aiff"},                  {"aif", "aifc", "aiff"}}, 
         {  {"audio/x-pn-realaudio"},          {"ra", "ram"}}, 
         {  {"audio/x-pn-realaudio-plugin"},   {""}}, 
         {  {"audio/x-realaudio"},             {"ra"}}, 
         {  {"audio/x-wav"},                   {"wav"}}, 
         {  {"chemical/x-pdb"},                {"pdb", "xyz"}}, 
         {  {"image/gif"},                     {"gif"}}, 
         {  {"image/ief"},                     {"ief"}}, 
         {  {"image/jpeg"},                    {"jpg", "jpeg", "jpe"}}, 
         {  {"image/png"},                     {"png"}}, 
         {  {"image/tiff"},                    {"tif", "tiff"}}, 
         {  {"image/x-cmu-raster"},            {"ras"}}, 
         {  {"image/x-portable-anymap"},       {"pnm"}}, 
         {  {"image/x-portable-bitmap"},       {"pbm"}}, 
         {  {"image/x-portable-graymap"},      {"pgm"}}, 
         {  {"image/x-portable-pixmap"},       {"ppm"}}, 
         {  {"image/x-rgb"},                   {"rgb"}}, 
         {  {"image/x-xbitmap"},               {"xbm"}}, 
         {  {"image/x-xpixmap"},               {"xpm"}}, 
         {  {"image/x-xwindowdump"},           {"xwd"}}, 
         {  {"message/external-body"},         {""}}, 
         {  {"message/news"},                  {""}}, 
         {  {"message/partial"},               {""}}, 
         {  {"message/rfc822"},                {""}}, 
         {  {"model/iges"},                    {"igs", "iges"}}, 
         {  {"model/mesh"},                    {"msh", "mesh", "silo"}}, 
         {  {"model/vrml"},                    {"wrl", "vrml"}}, 
         {  {"multipart/alternative"},         {""}}, 
         {  {"multipart/appledouble"},         {""}}, 
         {  {"multipart/digest"},              {""}}, 
         {  {"multipart/mixed"},               {""}}, 
         {  {"multipart/parallel"},            {""}}, 
         {  {"text/css"},                      {"css"}}, 
         {  {"text/html"},                     {"html", "htm"}}, 
         {  {"text/plain"},                    {"txt", "asc", "c", "cc", "h", "hh", "cpp", "hpp"}}, 
         {  {"text/richtext"},                 {"rtx"}}, 
         {  {"text/tab-separated-values"},     {"tsv"}}, 
         {  {"text/x-setext"},                 {"etx"}}, 
         {  {"text/x-sgml"},                   {"sgml", "sgm"}}, 
         {  {"text/x-vCalendar"},              {"vcs"}}, 
         {  {"text/x-vCard"},                  {"vcf"}}, 
         {  {"text/xml"},                      {"xml", "dtd"}}, 
         {  {"video/dl"},                      {"dl"}}, 
         {  {"video/fli"},                     {"fli"}}, 
         {  {"video/gl"},                      {"gl"}}, 
         {  {"video/mpeg"},                    {"mpeg", "mp2", "mpe", "mpg"}}, 
         {  {"video/quicktime"},               {"mov", "qt"}}, 
         {  {"video/x-msvideo"},               {"avi"}}, 
         {  {"video/x-sgi-movie"},             {"movie"}}, 
         {  {"x-conference/x-cooltalk"},       {"ice"}}, 
         {  {"x-world/x-vrml"},                {"vrml", "wrl"}}
      };




   /***************************************************************************
    * 
    **************************************************************************/
   public static String getMimeType(String filename)
   {
      String extension;
      int pos = 0;

      if(filename == null)
         return MIME_UNKNOWN;

      // remove leading and trailing white spaces (all smaller 0x20)
      // AWARE: tabs are not removed!
      filename = filename.trim(); 

      if ((pos = filename.lastIndexOf('.')) < 0)
         extension = filename;
      else
         extension = filename.substring(pos+1);

      if (extension.equals(EXTENSION_UNKNOWN)) {
         return MIME_UNKNOWN;
      }

      for (int i = 0; i < dataTable.length; i++) {
         String[] extensions = dataTable[i][1];
         for (int j = 0; j < extensions.length; j++) {
            if (extensions[j].equals(extension)) {
               return dataTable[i][0][0];
            }
         }
      }
      return MIME_UNKNOWN;
   }




   /***************************************************************************
    * This method returns all known file extension for the given 
    * mime type.
    *
    * @param      mimeType
    *             The mime type for which to search for all file extension, 
    *             e.g. "application/octet-stream".
    * @return     An array with all matching file extension e.g. 
    *             {"bin", "dms", "lha", "lzh", "exe", "class"}
    **************************************************************************/
   public static String[] getAllExtensions(String mimeType)
   {
      int pos = 0;

      if(mimeType == null)
         return EXTENSIONS_UNKNOWN;

      // remove leading and trailing white spaces (all smaller 0x20)
      // AWARE: tabs are not removed!
      mimeType = mimeType.trim(); 

      if ((pos = mimeType.indexOf('/')) < 0)
         return EXTENSIONS_UNKNOWN;

      for (int i = 0; i < dataTable.length; i++) {
         if (dataTable[i][0][0].equals(mimeType)) {
            return dataTable[i][1];
         }
      }
      return EXTENSIONS_UNKNOWN;
   }




   /***************************************************************************
    * This method returns the first known file extension for the given 
    * mime type.
    *
    * @param      mimeType
    *             The mime type for which to search for a file extension, 
    *             e.g. "application/octet-stream".
    * @return     The string with the file extension (not including the "." !),
    *             e.g. "bin"
    **************************************************************************/
   public static String getSingleExtension(String mimeType)
   {
      return MediaHelper.getAllExtensions(mimeType)[0];
   }




   /***************************************************************************
    * main<p />
    *
    * There are implemented some test cases.<br />
    * You can run all existing test after each modification at this class
    * by just typing:<p />
    * <code>   java org.jutils.media.MediaHelper</code>
    *
    * @ param     args
    *             All given parameter, non required.
    **************************************************************************/
   public static void main(String[] args) {

      System.out.println("\n");
      System.out.println("TEST getMimeType");
      System.out.println("----------------");
      System.out.println("getMimeType(\"c\") = '" + getMimeType("c") + "'");

      long lStart = System.currentTimeMillis();
      for (int i = 0; i < 10000; i++) {
         getMimeType("c");
      }
      long lStop = System.currentTimeMillis();
      long lValue = lStop - lStart;
      System.out.println("Lasttest: 10000 x getMimeType: " + lValue + " ms");


      System.out.println("\n");
      System.out.println("TEST getSingleExtension");
      System.out.println("-----------------------");
      System.out.println("getSingleExtension(\"text/html\") = '" + getSingleExtension("text/html") + "'");

      lStart = System.currentTimeMillis();
      for (int i = 0; i < 10000; i++) {
         getSingleExtension("text/html");
      }
      lStop = System.currentTimeMillis();
      lValue = lStop - lStart;
      System.out.println("Lasttest: 10000 x getSingleExtension: " + lValue + " ms");


      System.out.println("\n");
      System.out.println("TEST getAllExtensions");
      System.out.println("---------------------");
      System.out.println("getAllExtensions(\"text/html\"): Anzahl = '" + getAllExtensions("text/html").length + "'");

      lStart = System.currentTimeMillis();
      for (int i = 0; i < 10000; i++) {
         getAllExtensions("text/html");
      }
      lStop = System.currentTimeMillis();
      lValue = lStop - lStart;
      System.out.println("Lasttest: 10000 x getAllExtensions: " + lValue + " ms");


      System.out.println("\n");
   }
}
