package org.jutils.runtime;
/*------------------------------------------------------------------------------
Name:      StackTrace.java
Project:   jutils.org
Copyright: jutils.org, see jutils-LICENSE file
Comment:   
------------------------------------------------------------------------------*/


import java.io.StringWriter;
import java.io.PrintWriter;
import java.lang.StringBuffer;
import java.lang.Throwable;


/**
 * A Helper class to access the stack trace as string. 
 *
 * @author Marcel Ruff
 * @author Juergen Freidling
 */
public class StackTrace
{
   /**
    * Get the full stacktrace out of a Throwable (Exception and Error)
    * as string.<p />
    *
    * @param      t
    *             The throwable to trace.
    * @return     The full stacktrace as string.
    */
   public static final String getStackTrace(Throwable t)
   {
      StringWriter stringWriter = new StringWriter();
      StringBuffer writerBuffer = stringWriter.getBuffer();
      PrintWriter out = new PrintWriter(stringWriter, false);
      t.printStackTrace(out);
      out.flush();
      return writerBuffer.toString();
   }


   /**
    * Display the current stack<p />
    * 
    * @return     A full stacktrace of the current stack as string.
    */
   public static final String getStackTrace()
   {
      Throwable throwable = new Throwable();
      return getStackTrace(throwable);
   }
}
