package org.jutils.text;
/*------------------------------------------------------------------------------
Name:      LinkExtractor.java
Project:   jutils.org
Copyright: jutils.org, see jutils-LICENSE file
Comment:   The named logging instance which delegates the logging task
           to all registered logging devices in this object.
------------------------------------------------------------------------------*/


import java.io.*;
import java.net.*;
import javax.swing.text.*;
import javax.swing.text.html.*;


/*******************************************************************************
 * This is a very special helper class for searching and extracting links 
 * out of HTML pages. The HTML pages which should be analyzed could be a
 * local file or an remote URL request.<p />
 *
 * The following examples shows how to start this program and shows the 
 * output:<p />
 * 
 * <pre>
 * java org.jutils.text.LinkExtractor http://w3c.org/index.html 
 *
 * http://w3c.org/index.html : http://www.w3.org/
 * http://w3c.org/index.html : http://www.w3.org/History/
 * http://w3c.org/index.html : /Out-Of-Date/index.html
 * http://w3c.org/index.html : http://www.w3.org/
 * http://w3c.org/index.html : http://search.w3.org/
 * http://w3c.org/index.html : http://www.w3.org/Help/Webmaster
 * http://w3c.org/index.html : http://www.w3.org/Help/Webmaster
 * </pre>
 * 
 * @author Wolfgang Kleinertz
 ******************************************************************************/
public class LinkExtractor
{
   /****************************************************************************
    * No instance of this object should be created.
    ***************************************************************************/
   private LinkExtractor() 
   {
   }


   /****************************************************************************
    * @param      uri
    *             
    ***************************************************************************/
   private static Reader getReader(String uri) 
      throws IOException
   {
      if (uri.startsWith("http:")) {
         URLConnection conn = new URL(uri).openConnection();
         return new InputStreamReader(conn.getInputStream());
      } else {
         return new FileReader(uri);
      }
   }


   /****************************************************************************
    * 
    * 
    * java org.jutils.text.LinkExtractor uri
    ***************************************************************************/
   public static void main(String[] args)
   {
      EditorKit kit = new HTMLEditorKit();
      Document doc = kit.createDefaultDocument();

      doc.putProperty("IgnoreCharsetDirective", Boolean.TRUE);
      try {
         Reader rd = getReader(args[0]);

         kit.read(rd, doc, 0);

         ElementIterator it = new ElementIterator(doc);
         javax.swing.text.Element elem;
         while ((elem = it.next()) != null) {
            SimpleAttributeSet s = (SimpleAttributeSet)
            elem.getAttributes().getAttribute(HTML.Tag.A);
            if (s != null) {
               System.out.println(args[0] + " : " + 
                  s.getAttribute(HTML.Attribute.HREF));
            }
         }
      } catch (Exception e) {
         e.printStackTrace();
      }
      System.exit(1);
   }
}
