package org.jutils.text;
import java.util.*;

/**
 * Some utilities to manipulate strings.
 * @author Marcel Ruff
 */
public class StringHelper {
private final static String ME = "StringHelper";



   /************************************************************
    Diese Methode dient dem Hinzufgen eines Stringwertes zu einer
    String-Arrays. <br>

    @author Konrad Krafft
    @author Michael Winkler
    @param stringArray   Array ohne neuen Wert
    @param value         String, der hinzugefgt werden soll
    @return             Array mit neuem Wert
    @version gepflegt in cvs
   ************************************************************/
   public static final String[] addToStringArray(String[] stringArray, String value) {
     String[] returnValue = null;
     returnValue = addToStringArray(stringArray, value, null);
     return returnValue;
   }



   /************************************************************
    Diese Methode dient dem Hinzufgen eines Stringwertes zu einer
    String-Arrays. <br>

    @author Konrad Krafft
    @author Michael Winkler
    @param stringArray   Array ohne neuen Wert
    @param wert         String, der hinzugefgt werden soll
    @param destArray    The array to which the result should be copied. Will be returned. If nuill, the created by the method.
    @return             Array mit neuem Wert
    @version gepflegt in cvs
   ************************************************************/
   public static final String[] addToStringArray(String[] stringArray, String value, String[] destArray) {
     String[] returnValue = destArray;
     if (returnValue == null)
       returnValue = new String[stringArray.length + 1];

     // check wether this value laready is in the list
     for (int i = 0; i < stringArray.length; i++) {
       if (stringArray[i].equals(value)) {
         throw new IllegalArgumentException("Value '" + value + "' already present in String array!");
       }
     }
     System.arraycopy(stringArray, 0, returnValue, 0, stringArray.length);
     returnValue[returnValue.length - 1] = value;
     return returnValue;
   }



   /**
   * Modifies the passed string so that it is leaded by a chainn (of at least one) of leader chars.
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return java.lang.String
   * @param originalString java.lang.String
   * @param leader java.lang.String
   * @param resultLength int
   */
   public static String adjustLeading(String originalString, char leader, int resultLength) {
     if ((originalString == null) || (originalString.length() > resultLength))
       throw new IllegalArgumentException();
     StringBuffer returnValue = new StringBuffer(resultLength);
     int chainCount = resultLength - originalString.length();
     if (chainCount > 0) {
       for (int index = 0; index < chainCount; index++) {
         returnValue.append(leader);
       }
     }
     returnValue.append(originalString);
     return returnValue.toString();
   }



   /**
   * returns a String that contains <tt>count</tt> time the passed character
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return java.lang.String
   * @param oneChar char
   * @param count int
   */
   public String charChain(char oneChar, int count) {
     StringBuffer returnValue = new StringBuffer();
     if (count > 0)
       for (int index = 0; index < count; index++) {
         returnValue.append(oneChar);
       }
     return returnValue.toString();
   }



   /**
   * Clones the passed String array. String objects are immutable; therefore
   * there is no sence in cloning them too.
   * @author Michael Winkler, doubleSlash GmbH
   * @return A clone of the array. The contained Strings are NOT cloned.
   * @param originalArray java.lang.String[]
   */
   public static Object clone(String[] originalArray) {
     String[] returnValue = null;
     if (originalArray != null) {
       returnValue = new String[originalArray.length];
       System.arraycopy(originalArray, 0, returnValue, 0, originalArray.length);
     }
     return returnValue;
   }



   /**
   * Checks wether or not the passed String array contains the passed String.
   * @return boolean
   * @param stringArray The searched string array. may contain nulls, but may not BE null.
   * @param value The value to be searched.
   */
   public static boolean contains(String[] stringArray, String value) {
     return contains(stringArray, value, false);
   }



   /**
   * Checks wether or not the passed String array contains the passed String.
   * @return boolean
   * @param stringArray The searched string array. may contain nulls, but may not BE null.
   * @param value The value to be searched.
   * @param terminateOnNull Defines wether or not null strings may be used as a terminator sign..
   */
   public static boolean contains(String[] stringArray, String value, boolean terminateOnNull) {
     if (stringArray == null)
       throw new IllegalArgumentException("Cannot search null pointer arrays!");
     boolean returnValue = false;
     int index = getFirstIndex(stringArray, value, terminateOnNull);
     returnValue = index > -1;
     return returnValue;
   }



   /**
   * Checks wether or not the passed String
   * array contains the passed String, and
   * returns the index of that value, if found.
   * @return int The position in the array, if found. A negative value, which is corresponding to the position in the array on which the search has been ended, is indicating that the value has NOT been found.
   * @param stringArray The searched string array. may contain nulls, but may not BE null.
   * @param value The value to be searched. May NOT be null, for now ...
   * @param terminateOnNull Defines wether or not null strings may be used as a terminator sign. Useful when searching dynamic arrays.
   */
   public static int getFirstIndex(String[] stringArray, String value, boolean terminateOnNull) {
     if (stringArray == null)
       throw new IllegalArgumentException("Casnnot search null pointer arrays!");
     int returnValue = stringArray.length * -1;
     int index = 0;
     boolean terminateConditionMet = index >= stringArray.length;
     while (!terminateConditionMet) {

       // is the value in the string array == null?!?
       if (stringArray[index] != null) {
         if ((stringArray[index] == value) || stringArray[index].equals(value)) {
            returnValue = index;
            terminateConditionMet = true;
         }
       }
       else
         if (terminateOnNull) {
            terminateConditionMet = true;
            returnValue = index;
         }

         // advance index
       if (!terminateConditionMet) {
         index++;
         terminateConditionMet = index >= stringArray.length;
       }
     }
     return returnValue;
   }



   /**
   * For testing only
   * <p />
   * java org.jutils.util.StringHelper
   */
   public static void main(String args[]) {

     // Check String manipulating methods
     String ME = "StringHelper";
     String input = "Hello my friend, my dear friend";
     System.out.println(
       "Converting '"
         + input
         + "' to (one replacement):\n"
         + "'"
         + StringHelper.replaceFirst(input, "friend", "Jacqueline")
         + "'");
     System.out.println(
       "Converting '"
         + input
         + "' to (all replacement):\n"
         + "'"
         + StringHelper.replaceAll(input, "friend", "Jacqueline")
         + "'");
     input = "Hello my friend, my dear friend";
     System.out.println(
       "Converting '" + input + "' to (one replacement):\n" + "'" + StringHelper.replaceFirst(input, "H", "") + "'");
     System.out.println(
       "Converting '" + input + "' to (all replacement):\n" + "'" + StringHelper.replaceAll(input, "H", "") + "'");
     input = "Hello my friend, my dear friend";
     System.out.println(
       "Converting '" + input + "' to (one replacement):\n" + "'" + StringHelper.replaceFirst(input, null, "") + "'");
     System.out.println(
       "Converting '" + input + "' to (all replacement):\n" + "'" + StringHelper.replaceAll(input, null, "") + "'");
     input = "Hello my friend, my dear friend";
     System.out.println(
       "Converting '" + input + "' to (one replacement):\n" + "'" + StringHelper.replaceFirst(input, "#", "Jacqueline") + "'");
     System.out.println(
       "Converting '" + input + "' to (all replacement):\n" + "'" + StringHelper.replaceAll(input, "#", "Jacqueline") + "'");
     input = "Hello my friend, my dear friend";
     System.out.println(
       "Converting '" + input + "' to (one replacement):\n" + "'" + StringHelper.replaceFirst(input, "l", "&l;") + "'");
     System.out.println(
       "Converting '" + input + "' to (all replacement):\n" + "'" + StringHelper.replaceAll(input, "l", "&l;") + "'");
     input = " ";
     System.out.println(
       "Converting '" + input + "' to (one replacement):\n" + "'" + StringHelper.replaceFirst(input, "l", "&l;") + "'");
     System.out.println(
       "Converting '" + input + "' to (all replacement):\n" + "'" + StringHelper.replaceAll(input, "l", "&l;") + "'");

     // check String array manipulating methods
     int errorCount = 0;
     System.out.println("==============================================================");
     System.out.println("Testing methods for manipulation of String arrays ...");
     System.out.println("--------------------------------------------------------------");
     System.out.println("Initializing ...");
     int arraySize = 1000;
     String[] originalArray = new String[arraySize];
     for (int index = 0; index < arraySize; index++) {
       originalArray[index] = String.valueOf(index);
     }
     System.out.println("Checking clone ...");
     long startTicks = System.currentTimeMillis();
     String[] clone = (String[]) clone(originalArray);
     long stopTicks = System.currentTimeMillis();
     System.out.println("Time needed to clone: " + String.valueOf(stopTicks - startTicks) + " ms.");
     for (int index = 0; index < originalArray.length; index++) {
       if (originalArray[index] != clone[index]) {
         System.err.println("error detected in clone at element " + index + "!");
         errorCount++;
       }
     }

     // first, check if getFirstIndex detects start and stop of the array correctly
     int index = getFirstIndex(clone, clone[0], false);
     if (index != 0) {
       System.err.println("asking getFirstIndex for the first element did return wrong index! (" + index + ")");
       errorCount++;
     }
     index = getFirstIndex(clone, clone[clone.length - 1], false);
     if (index != clone.length - 1) {
       System.err.println("asking getFirstIndex for the last element did return wrong index! (" + index + ")");
       errorCount++;
     }

     // randomly check 10% of the contents
     int elementsToCheck = Math.max(clone.length / 10, Math.min(clone.length, 10));
     System.out.println("Randomly checking " + elementsToCheck + " elements...");
     for (int counter = 0; counter < elementsToCheck; counter++) {
       index = (int) (originalArray.length * Math.random());
       if (originalArray[index] != clone[index]) {
         System.err.println("Error at position " + index + " !");
         errorCount++;
       }
     }

     // Check addToStringArray
     System.out.println("Checking addToStringArray ...");
     String valueToAdd = String.valueOf(arraySize);
     startTicks = System.currentTimeMillis();
     clone = addToStringArray(clone, valueToAdd);
     stopTicks = System.currentTimeMillis();
     System.out.println("Time needed to add : " + String.valueOf(stopTicks - startTicks) + " ms.");

     // check if the element really has been added...
     if (clone[clone.length - 1] != valueToAdd) {
       System.err.println("'addToStringArray' did not append the specified element!");
       errorCount++;
     }

     // Check removeFromStringArray
     System.out.println("Checking removeFromStringArray...");
     startTicks = System.currentTimeMillis();
     clone = removeFromStringArray(clone, valueToAdd);
     stopTicks = System.currentTimeMillis();
     System.out.println("Time needed to remove : " + String.valueOf(stopTicks - startTicks) + " ms.");

     // check if the element really has been added...
     if (getFirstIndex(clone, valueToAdd, false) > -1) {
       System.err.println("'removeFromStringArray' did not append the specified element!");
       errorCount++;
     }

     // check htmlEncode
     System.out.println("Testing htmlEncode() ...");
     String string = "We went to the \uudcberholspur and were <marvellously> refreshed!";
     string = htmlEncode(string);
     System.out.println(string);
     System.out.println("--------------------------------------------------------------");
     System.out.println(String.valueOf(errorCount) + " errors detected.");
     System.out.println("==============================================================");
     System.exit(errorCount);
   }



   /************************************************************
    Diese Methode dient dem Entfernen eines Stringwertes von einer
    String-Liste. <br>

    @author Konrad Krafft
    @author Michael Winkler
    @param stringArray   Array mit zu entfernenden Wert
    @param value         String, der entfernt werden soll
    @param value         String, der entfernt werden soll
    @return             Array ohne zu entfernden Wert
    @version gepflegt in cvs
   ************************************************************/
   public static final String[] removeFromStringArray(String[] stringArray, String value) {
     String[] returnValue = null;
     returnValue = removeFromStringArray(stringArray, value, false, true);
     return returnValue;
   }



   /************************************************************
    Diese Methode dient dem Entfernen eines Stringwertes von einer
    String-Liste. <br>

    @author Konrad Krafft
    @author Michael Winkler
    @param stringArray   Array mit zu entfernenden Wert
    @param wert         String, der entfernt werden soll
    @param terminateOnNull Defines that the search should be ended on the first null pointer in the array.
    @return             Array ohne zu entfernden Wert
    @version gepflegt in cvs
   ************************************************************/
   public static final String[] removeFromStringArray(String[] stringArray, String value, boolean terminateOnNull) {
     String[] returnValue = null;
     returnValue = removeFromStringArray(stringArray, value, terminateOnNull, true);
     return returnValue;
   }



   /************************************************************
    Diese Methode dient dem Entfernen eines Stringwertes von einer
    String-Liste. <br>

    @author Konrad Krafft
    @author Michael Winkler
    @param stringArray   Array mit zu entfernenden Wert
    @param wert         String, der entfernt werden soll
    @param terminateOnNull Defines that the search should be ended on the first null pointer in the array.
    @return             Array ohne zu entfernden Wert
    @version gepflegt in cvs
   ************************************************************/
   public static final String[] removeFromStringArray(String[] stringArray, String value, boolean terminateOnNull, boolean copyAfterNull) {
     String[] returnValue = null;
     returnValue = removeFromStringArray(stringArray, value, terminateOnNull, copyAfterNull, null);
     return returnValue;
   }



   /************************************************************
    Diese Methode dient dem Entfernen eines Stringwertes von einer
    String-Liste. <br>

    @author Konrad Krafft
    @author Michael Winkler
    @param stringArray           Array mit zu entfernenden Wert
    @param value                 String, der entfernt werden soll
    @param terminateOnNull     Defines that the search should be ended on the first null pointer in the array.
    @param destArray           The array to which the result should be copied. If null, the array will be created.
    @return                     The resulting array. If destArray != null, then returnvalue == destArray.
    @version gepflegt in cvs
   ************************************************************/
   public static final String[] removeFromStringArray(String[] stringArray, String value, boolean terminateOnNull, boolean copyAfterNull, String[] destArray) {


     // check params
     // --------------------------------------------------------------

     if (stringArray == null) {
       throw new IllegalArgumentException("You cannot handle a null pointer array!");
     }

     // check the returnValue
     if ((destArray != null) && (destArray.length < stringArray.length - 1))
       throw new IllegalArgumentException("Cannot copy to an array with a length smaller that the original arrays length - 1!");

     // check the value
     if (value == null)
       throw new IllegalArgumentException("Cannot handle null values - yet!");

     // setup return value
     String[] returnValue = destArray;

     // find string
     int index = getFirstIndex(stringArray, value, terminateOnNull);

     // if found, copy the contents to the new location
     if (index > -1) {
       if (returnValue == null)
         returnValue = new String[stringArray.length - 1]; // ...assuming that the element has been found ...

       // copy part "before" index, if present
       if (index > 0)
         System.arraycopy(stringArray, 0, returnValue, 0, index - 1);

       // copy part "after" index, if present
       if (index < returnValue.length - 1)
         System.arraycopy(stringArray, index + 1, returnValue, index, returnValue.length - index - 1);
     }
     else {
       // copy the data to the new array, if it's different (i.e. another reference) from the old array
       if (returnValue != stringArray) {
         if (returnValue == null)
            returnValue = new String[stringArray.length];
         System.arraycopy(stringArray, 0, returnValue, 0, stringArray.length);
       }
       // If its the same difference, and no occurence has been found, do nothing.
     }
     return returnValue;
   }



   /**
   * Replace all occurrences of "from" with to "to".
   */
   public final static String replaceAll(String str, String from, String to) {
     if (str == null || str.length() < 1 || from == null || to == null)
       return str;
     if (str.indexOf(from) == -1)
       return str;
     StringBuffer buf = new StringBuffer("");
     String tail = str;
     while (true) {
       int index = tail.indexOf(from);
       if (index >= 0) {
         if (index > 0)
            buf.append(tail.substring(0, index));
         buf.append(to);
         tail = tail.substring(index + from.length());
       }
       else
         break;
     }
     buf.append(tail);
     return buf.toString();
   }



   /**
   * Replace exactly one occurrence of "from" with to "to"
   */
   public final static String replaceFirst(String str, String from, String to) {
     if (str == null || str.length() < 1 || from == null || to == null)
       return str;
     int index = str.indexOf(from);
     if (index >= 0) {
       StringBuffer tmp = new StringBuffer("");
       if (index > 0)
         tmp.append(str.substring(0, index));
       tmp.append(to);
       tmp.append(str.substring(index + from.length()));
       return tmp.toString();
     }
     else
       return str;
   }



   /**
   * Create a more representative string representation for an object array.
   **/
   public static String toString(Object[] array) {
     StringBuffer returnValue = new StringBuffer();
     if (array != null) {
       if (array.length > 0) {
         returnValue.append('[');
         for (int index = 0; index < array.length; index++) {
            if (index > 0)
              returnValue.append(',');
            returnValue.append(array[index]);
         }
         returnValue.append(']');

       } else
         returnValue.append("[]");
     } else
       returnValue.append("null");
     return returnValue.toString();

   }



   /**
   * Converts this string to a html string, i.e.
   * replaces all special chars like the german umlauts an &gt; and &lt; with the
   * corresponding html tag.<br>
   * <strong>CAUTION!</strong>
   * Do NOT use when some of the characters are NECESSARY for data structure.
   * For example, do NOT encode an xml stream if you intend to use its data.
   * Creation date: (16.11.2000 11:42:37)
   * @return java.lang.String the html-encoded string
   * @param string java.lang.String a String to HTML-Encode.
   */
   public static String htmlEncode(String string) {

     // MW HACK
     string= replaceAll(string, "#", " ");
     string= replaceAll(string, "&", "+");
     string= replaceAll(string, " ", " ");
     string= replaceAll(string, "%", " ");
     string= replaceAll(string, "<", " ");
     string= replaceAll(string, ">", " ");

     string= replaceAll(string, "\u00e4", "ae");
     string= replaceAll(string, "\u00f6", "oe");
     string= replaceAll(string, "\u00fc", "ue");
     char[] charArray= string.toCharArray();

     /*
     string = replaceAll(string, "&", "&amp;");
     string = replaceAll(string, "#", "&#35;");
     string = replaceAll(string, " ", "&nbsp;");
     string = replaceAll(string, "%", "&#37;");
     string = replaceAll(string, "<", "&lt;");
     string = replaceAll(string, ">", "&gt;");
     string = replaceAll(string, "\u00c4", "&Auml;");
     string = replaceAll(string, "\u00e4", "&auml;");
     string = replaceAll(string, "\u00d6", "&Ouml;");
     string = replaceAll(string, "\u00f6", "&ouml;");
     string = replaceAll(string, "\u00dc", "&Uuml;");
     string = replaceAll(string, "\u00fc", "&uuml;");
     */

     // return java.net.URLEncoder.encode(string);
     return string;
   }



/**
 * Checks wether or not all values of <code>valuesToFind</code> are part of <code>stringToScan</code>.
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @return boolean
 * @param stringToScan java.lang.String
 * @param valuesToFind java.util.List
 */
public static boolean contailsAll(String stringToScan, List valuesToFind) {
   Iterator iterator = valuesToFind.iterator();
   boolean returnValue = iterator.hasNext();
   while (returnValue && iterator.hasNext())
     returnValue = stringToScan.indexOf(iterator.next().toString()) > -1;
   return returnValue;
}





/**
 * Counts the occurrences of <code>stringToCount</code> in <code>stringToScan</code>.
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @return int
 * @param stringToScan java.lang.String The string to scan that contains none or some occurrences of <code>stringToCount</code>.
 * @param stringToCount java.lang.String The string to look for in <code>stringToScan</code>.
 * @param caseSensitive boolean Wether or not the case of the letters counts.
 */
public static int countOccurrence(String stringToScan, String stringToCount, boolean caseSensitive) {
   int returnValue = 0;

   // check for case insensitivity
   if (!caseSensitive) {
      stringToScan = stringToScan.toLowerCase();
      stringToCount = stringToCount.toLowerCase();
   }

   // count the occurrences.
   int lastPosition = stringToScan.indexOf(stringToCount);
   while (lastPosition > -1) {
      returnValue++;
      lastPosition = stringToScan.indexOf(stringToCount, lastPosition + 1);
   }
   return returnValue;
}
}
