package org.jutils.time;

/*------------------------------------------------------------------------------
Name:      DateTime.java
Project:   jutils.org
Copyright: jutils.org, see jutils-LICENSE file
Comment:   Some Date formatting
Version:   $Id: DateTime.java,v 1.4 2001/08/29 13:25:55 obelikan Exp $
------------------------------------------------------------------------------*/
import java.util.Locale;
import java.util.Date;
import java.text.*;
import java.util.Locale;
import org.jutils.JUtilsException;

/**
 * represent DateTime as pretty formattet string
 */
public class DateTime {

   static public final String DATE     = "date";
   static public final String FROMDATE = "fromdate";
   static public final String TODATE   = "todate";

   private Date myDate     = new Date();
   private String thisDate = null;
   private String year     = "2000";
   private String month    = "01";
   private String day      = "01";
   private String hour     = "00";
   private String minute   = "00";
   private String second   = "00";
   private String datetimetype = DateTime.DATE;
   private SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy hh:mm", Locale.GERMANY);


/**
 *
 */
   public DateTime () {
      myDate = new Date();
      this.setDateElements();
   }

   /**
    * insert date like dd.MM.yyyy hh:mm
    */
   public DateTime (String thedatetime) throws JUtilsException  {
      this.setDateTime (thedatetime);
      this.setDateElements();
   }

   /**
    * insert date like dd.mm.yyyy hh:mm
    */
   public DateTime (long thedatetime) {
      myDate = new Date(thedatetime);
      this.thisDate = this.sdf.format(myDate);
      this.setDateElements();
   }

   /**
    * insert date like
    * @param format possible values are
    * 'dd.mm.yyyy hh:mm' or 'dd.mm.yyyy'
    */
   public DateTime (String thedatetime, String format) throws JUtilsException {
      sdf = new SimpleDateFormat (format, Locale.GERMAN);
      this.setDateTime (thedatetime);
      this.setDateElements();
   }
   /**
    * sets the type of Datetime.
    * Default is DateTime.DATE
    * Possible DateTime.FROMDATE or DateTime.TODATE
    */
    public void setType (String datetimetype) {
      if (datetimetype.equals(DateTime.DATE))
         this.datetimetype = DateTime.DATE;
      if (datetimetype.equals(DateTime.FROMDATE))
         this.datetimetype = DateTime.FROMDATE;
      if (datetimetype.equals(DateTime.TODATE))
         this.datetimetype = DateTime.TODATE;
    }

    /**
   *
   */
   public Date getDate() {
      return this.myDate;
   }

   /**
    *
    */
    public String getMonth() {
      return month;
    }

    /**
     *
     */
    public String getDay() {
      return day;
    }

   /**
    *
    */
    public String getYear() {
      return year;

    }
    /**
     *
     */
   public String getHour() {
      return hour;
   }
   /**
    *
    */
   public String getMinute() {
      return minute;
   }

   /**
    *
    */
    public String toString() {
       return getAsString();
    }
    /**
     *
     */
     public String getAsString () {
        return thisDate;
     }
    /**
     *
     */
   public String getSecond() {
      return second;
   }
   /**
    *
    */
   public void setSecond (String second) {
      this.second = second;
   }

     /**
      *
      */
      public String getAsXml() {
         String retStr = "" +
            "<"+this.datetimetype+">\n"+
            "   <day>" + day +"</day>\n"+
            "   <month>" + month +"</month>\n"+
            "   <year>" + year+"</year>\n"+
            "   <hour>" + hour + "</hour>\n"+
            "   <minute>" + minute + "</minute>\n"+
            "   <second>" + second + "</second>\n"+
            "</"+this.datetimetype+">\n";
           return retStr;
      }

    /**
     * @param  Datestring as dd.MM.yyyy hh:mm
     */
     private void setDateTime (String mydatetime) {
           ParsePosition pos = new ParsePosition(0);
           myDate = sdf.parse (mydatetime, pos);
           this.thisDate = this.sdf.format(myDate);
     }

     /**
      * @param Returns the numver of milliseconds since 01.01.1970 00:00:00
      */
      public long getTimeInMillis() {
         return this.getDate().getTime();
      }


     /**
      * sets the elements day, month, year, hour, minute, second....
      */
   private void setDateElements() {
      SimpleDateFormat mySdf = new SimpleDateFormat ("dd.MM.yyyy hh:mm:ss", Locale.GERMANY);
      String thatDate = mySdf.format(myDate);  // dd.MM.yyyy hh:mm:ss
      day    = thatDate.substring(0, thatDate.indexOf("."));
      month  = thatDate.substring (thatDate.indexOf(".")+1, thatDate.lastIndexOf("."));
      year   = thatDate.substring (thatDate.lastIndexOf(".")+1, thatDate.lastIndexOf(".")+5);
      hour   = thatDate.substring (thatDate.indexOf(":")-2, thatDate.indexOf(":"));
      minute = thatDate.substring (thatDate.indexOf(":")+1, thatDate.indexOf(":")+3);
      second = thatDate.substring (thatDate.indexOf(":")+4, thatDate.indexOf(":")+6);
   }

   /**
   */
   public static void main(String args[]) throws Exception {
      try {
      String ME = "DateTime-Tester";

      DateTime to = new DateTime("12.12.1980 00:01");
      DateTime from   = new DateTime("12.12.1980 00:00");
      DateTime fromm  = new DateTime(System.currentTimeMillis());

      System.out.println ("from=" + from.getAsString() + "  to="+to.getAsString());

      System.out.println ("from=" + from.getDate().getTime() + "  to="+to.getDate().getTime());

      System.out.println(from.getAsXml());
      System.out.println(to.getAsXml());
      System.out.println(fromm.getAsString());
      }
      catch (JUtilsException e) {
         System.out.println(e.reason);
      }
   }
}
